/**
 * GenericTestBed.java
 * 
 * Time-stamp: <2005-02-15 17:12:11 ftaiani>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: GenericTestBed.java,v 1.2 2005/02/15 17:12:20 ftaiani Exp $
 * 
 */

/**
 * This abstract class provides the basic interface to implement a
 * test bed class. To use it, you need to subclass it and define the
 * method {@link #aNewSecondHasPassed()}. Your implementation of
 * {@link #aNewSecondHasPassed()} should contain the code you want to
 * be executed by the test bed each time the control loop of the
 * insulin pump (in Class {@link Clock}) has completed one cycle. This
 * happens every second (Note that 'seconds' might be speeded up see
 * {@link #setDurationOfOneSecondInMS(int)}).<p>
 *
 * Here is an example on how to use this class to define your own test
 * bed class:<p>
 *
 * <pre>
 *public class ToyTestBed extends GenericTestBed {
 *
 *    public void aNewSecondHasPassed() {
 *	if (getSimulationTime()==5)
 *	    hardware.removeInsulinReservoir() ;
 *	if (getSimulationTime()==7)
 *	    display.pushButtonAdministerInsulin(4) ;
 *	if (getSimulationTime()>10)
 *	    destroyInsulinPump();
 *    } // EndMethod aNewSecondHasPassed
 *
 *    public static void main(String[] args) {
 * 	ToyTestBed testBed = new ToyTestBed() ;
 *	testBed.setDurationOfOneSecondInMS(500); // twice faster
 *	testBed.launchNewInsulinPump() ;	    
 *	System.exit(0);
 *    } // EndMain
 *
 *} // EndClass ToyTestBed
 *</pre>
 *
 * In this example the class {@link ToyTestBed} is subclassed from
 * {@link GenericTestBed}. It can be launched by usind <code>java
 * ToyTestBed</code> on the command line.<p>
 *
 * When launched, <code>main</code> executes first and creates a new
 * <code>ToyTestBed</code> object. It sets the duration of one
 * simulation 'second' to 500 ms (resulting in a 2x speedup:
 *<pre>
 * testBed.setDurationOfOneSecondInMS(500) ;
 *</pre>
 *
 * The insulin pump is then launched inside the testbed using 
 *<pre>
 * testBed.launchNewInsulinPump() ;
 *</pre>
 *
 * After this line the control is handed to the insulin pump software,
 * which regularly calls back the method {@link
 * #aNewSecondHasPassed()} of {@link ToyTestBed}. In {@link
 * #aNewSecondHasPassed()}, {@link ToyTestBed} checks is the insulin
 * pump execution has reached some particular deadline. If yes it
 * triggers some action using the {@link #hardware} and {@link
 * #display} objects. For instance, after 5 seconds the insulin
 * reservoir is removed. After 7 second, the test bed request 4 doses
 * of insulin to be injected, and after 10 seconds, the insulin pump
 * software is stopped.<p>
 *
 * Note that this example is very simple and does not use the {@link
 * #display} to read displays. Have a look at the source of {@link
 * SampleTestBed} for a more complete example of how a safety property
 * can be tested by using the {@link GenericTestBed} class.
 *
 * @author Francois Taiani   <f.taiani@computer.org>
 */
public abstract class GenericTestBed {

    protected HardwareSimulatorAPI hardware = new HardwareSimulatorAPI(this) ;
    protected DisplaySimulatorAPI  display  = new DisplaySimulatorAPI(this) ;

    private static GenericTestBed activeTestBedObject = null ;
    
    public static final GenericTestBed getActiveTestBedObject() {
	return activeTestBedObject ;
    }

    public GenericTestBed() {
	activeTestBedObject = this ;
    } // EndConstructor GenericTestBed

    /**
     * Creates a new insulin pump and starts executing it.Only one
     * insulin pump is allowed at a time. Every second of the
     * execution time ('seconds' might be speeded up see {@link
     * #setDurationOfOneSecondInMS(int)}), the method {@link
     * #aNewSecondHasPassed()} will be called.
     * @see #setDurationOfOneSecondInMS(int)
     * @see #aNewSecondHasPassed()
     */
    public void launchNewInsulinPump() {
	resetAllStaticVariable();
	new Display().setVisible(true);
	hardware.startInsulinPumpController();
    }

    /** 
     * This method is called each time the {@link Clock} class
     * completes a control loop cycle. In this method you can use the
     * hardware simulation interface {@link #hardware} to simulate
     * failures, the display simulation interface {@link #display} to
     * activate the human computer interface and read the displays.<p>
     *
     * {@link GenericTestBed} provides a number of 'helper' methods
     * (listed below) that you can use inside {@link
     * #aNewSecondHasPassed()} to get information on the simulation,
     * print messages, and to modify some parameters of the execution.
     *
     * Note that the pump will execute until {@link
     * #destroyInsulinPump()} is called. If {@link
     * #destroyInsulinPump()} is not called inside {@link
     * #aNewSecondHasPassed()} the pump never stops.
     *
     * @see #getSimulationTime()
     * @see #printMessage(String)
     * @see #destroyInsulinPump()
     * @see #setDurationOfOneSecondInMS(int)
     */
    public abstract void aNewSecondHasPassed();

    /**
     * Stops the pump being executed and destroys it. Should be called
     * inside {@link #aNewSecondHasPassed()} to stop the test being
     * executed.
     */
    public void destroyInsulinPump() {
	hardware.stopInsulinPumpController() ;
	Display.closeInsulinDisplay();
    }

    /**
     * An helper method to get the time elapsed since the start of the
     * pump (in seconds. Note that 'seconds' might be speeded up see
     * {@link #setDurationOfOneSecondInMS(int)}).
     * @see #setDurationOfOneSecondInMS(int)
     */
    public int getSimulationTime() {
	return Controller.getClockTime() ;
    } // EndConstructor getSimulationTime

    /**
     * Internal use.
     */
    private void resetAllStaticVariable() {
	Controller.bigReinitHack();
	Display   .bigReinitHack();
	    
	NeedleAssembly  .setNeedlePresent(true);
	PowerSupply     .setBatteryLevel (true);
	Sensor          .setReading(0);
	Sensor          .setSensorWorking(true);
	InsulinPump     .setPumpWorking(true) ;
	InsulinPump     .deliverInsulin(0) ;
	InsulinReservoir.replaceInsulin() ;
    }

    /**
     * An helper method to pretty-print messages on the
     * console. Prefixes all messages with the current 'execution
     * time' in seconds.
     * @see #getSimulationTime()
     * @see #aNewSecondHasPassed()
     */
    public void printMessage(String msg) {
	System.err.println("("+getSimulationTime()+"): "+msg) ;
    } // EndMethod printMessage

    /** **************************************************** By
     * default one second lasts 1000ms in the controlling loop. If you
     * want the controller to run faster, set a value lower than
     * 1000ms. Setting the duration to 0 has the controller running as
     * fast as possible (best for testing).
     */
    public void setDurationOfOneSecondInMS(int duration) {
	Clock.clockCalibration = duration ;
    } // EndMethod setDurationOfOneSecondInMS

} // EndClass GenericTestBed
