/**
 * SampleTestBed.java
 * 
 * Time-stamp: <2005-04-04 11:53:32 ftaiani>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: SampleTestBed.java,v 1.3 2005/04/04 11:20:02 ftaiani Exp $
 * 
 */

/**
 * A sample test bed class. This class is based on {@link
 * GenericTestBed}. It checks the following safety property: after the
 * insulin reservoir has been removed for at least 20s, the display
 * should not tell the user that some insulin is remaining.<p>
 *
 * The safety property is implemented in the method {@link
 * #safetyPropertyIsRespected()}. This method is used in {@link
 * #aNewSecondHasPassed()} to check that the property is respected
 * after each second.<p>
 *
 * To test the insulin pump in different configuration, the test bed
 * launches 5 successive experiments (in {@link #runTestSet()}). In
 * each experiment a fixed amount of insulin is injected 5 second
 * after the start. The reservoir is removed at different time
 * depending on the experiment: 5s, 10s, 15s, 20s, 25s.<p>
 *
 * You can run this SampleTestBed by using<p>
 *
 * <code>java SampleTestBed</code>
 *
 * @author Francois Taiani   <f.taiani@computer.org>
 */
public class SampleTestBed extends GenericTestBed {

    public SampleTestBed() {
    } // EndConstructor SampleTestBed

    protected int timeWhenReservoirIsRemoved =  0 ;
    protected int timeWhenInsulinInjected    =  5 ;
    protected int durationOfExperience       = 30 ;
    protected int numberOfFailedExperience   =  0 ;

    public void aNewSecondHasPassed() {
	// printMessage("In SampleTestBed");
	
	// We switch to manual right at the beginning

	if (getSimulationTime()==1)
	    display.pushButtonOperatorModeChange();

	// We remove the reservoir if the time has come

	if (getSimulationTime()==timeWhenReservoirIsRemoved)
	    hardware.removeInsulinReservoir() ;
	
	// We inject some insulin if the time has come

	if (getSimulationTime()==timeWhenInsulinInjected) {
	    display.pushButtonAdministerInsulin(4) ;
	}
	
	// We check the safety property

	if (!safetyPropertyIsRespected()) {
	    printMessage("SAFERY PROPERTY VIOLATED!");
	    numberOfFailedExperience++;
	    destroyInsulinPump();
	}

	// If we have moved beyond the experience duration, we
	// terminate the experience
	
	if (getSimulationTime()>durationOfExperience)
	    destroyInsulinPump();

    } // EndMethod aNewSecondHasPassed

    protected int timeSinceReservoirHasBeenRemoved = 0 ;

    /** 
     * This method check the safety property targeted by the test bed:
     * "after the insulin reservoir has been removed for at least 20s,
     * the display should not tell the user that some insulin is
     * remaining".
     *
     * @return: true if the property is true, false otherwise.
     */
    protected boolean safetyPropertyIsRespected() {

	// We first count how long the reservoir has been removed. To
	// do that we first check if the reservoir is present. If it
	// is not, we increment
	// 'timeSinceReservoirHasBeenRemoved'. Otherwise we reset the
	// 'timeSinceReservoirHasBeenRemoved'.
	
	if (!hardware.insulinReservoirIsPresent())
	    timeSinceReservoirHasBeenRemoved++ ;
	else
	    timeSinceReservoirHasBeenRemoved = 0 ;

	// Now we check the real safety property: 'the display must
	// not tell us that there is still some insulin if the
	// reservoir is not present.' Since we must allow the
	// controller some time to detect that the controller is
	// missing, we check this property after the reservoir has
	// been removed for at least 20s.

	if (timeSinceReservoirHasBeenRemoved>=20) {

	    String whatTheDisplayTellsUs = display.readOutputDisplay() ;

	    if (whatTheDisplayTellsUs.startsWith("Insulin remaining = ")) {
		String insulinAmountOnDisplay = 
		    whatTheDisplayTellsUs.substring(20);
		int amount = Integer.parseInt(insulinAmountOnDisplay) ;
		return (amount==0) ; // Otherwise display is lying.
	    } // EndIf whatTheDisplayTellsUs.beginsWith("Insulin remaining = ")

	} // EndIf (timeSinceReservoirHasBeenRemoved>=20)

	return true ;

    } // EndMethod safetyPropertyIsRespected

    public void runTestSet() {
	for(timeWhenReservoirIsRemoved = 5 ; 
	    timeWhenReservoirIsRemoved < durationOfExperience ;
	    timeWhenReservoirIsRemoved += 5 ) {
	    
	    timeSinceReservoirHasBeenRemoved = 0 ;

	    this.launchNewInsulinPump() ;	    

	} // EndFor timeWhenReservoirIsRemoved

	System.err.println() ;
	System.err.print("RESULTS OF TESTING: ") ;

	if (numberOfFailedExperience==0)
	    System.err.println("SUCCESSFUL");
	else
	    System.err.println("FAILED "+numberOfFailedExperience+" time(s).");


    } // EndMethod runTestSet

    public static void main(String[] args) {
 	SampleTestBed testBed = new SampleTestBed() ;
	testBed.setDurationOfOneSecondInMS(50); // 20 times faster
	testBed.runTestSet() ;
	System.exit(0);
    } // EndMain

} // EndClass SampleTestBed
